import threading
import thread
import time
import string

# python2.1 compatibility
try:
    import True
except:
    True = 1
    False = 0


DEFAULT_TIMEOUT=600
SYSTEM_ENCODING = 'iso-8859-1'


class CacheObject:
    """ A simple cache object """

    def __init__(self, **kw):
        self.created = time.time()
        self.setData(kw)

    def setData(self, data):
        self.data = data

    def __getitem__(self, key):
        return self.data[key]

class RoleCache:
    """ class to cache ADS groups for a user object """

    def __init__(self, timeout=DEFAULT_TIMEOUT):
        self.timeout = timeout
        self.userinfo = None
        self._lock = threading.Lock()

    def _isValid(self, obj):
    	""" Check to see if the cached entry is valid """
        if obj is None:
            return False
        now = time.time()
        if obj.created + self.timeout > now :
            return True
        return False
        
    def setTimeout(self, timeout=DEFAULT_TIMEOUT):
    	""" Set the timeout (in seconds) for cached entries """
    	self.timeout = timeout

    def getUserInfo(self, default=None):
    	""" Returns cached roles for a user if they exist """
        val = self.userinfo
        if self._isValid(val):
            return val['data']
        self.userinfo = None
        return default

    def cacheUserInfo(self, data):
    	""" Caches the roles """
        self._lock.acquire()
        self.userinfo = CacheObject(data=data)
        self._lock.release()

    def invalidate(self):
    	""" Removes the cache entry for a user """
        self._lock.acquire()
        self.userinfo = None
        self._lock.release()

_cache = None

def getCache(timeout=DEFAULT_TIMEOUT):
    """ Returns a cache if it exists, otherwise creates one and returns it """
    global _cache
    if _cache is None:
        _cache = RoleCache(timeout)
    return _cache



